#include "server_socket.h"

/* Initialistion d'un socket serveur */
SOCKET server_socket_init() {
	SOCKET sock = socket(AF_INET, SOCK_STREAM, 0);

	if (sock == INVALID_SOCKET) {
		perror("socket()");
		exit(errno);
	}

	SOCKADDR_IN sin = { 0 };

	sin.sin_addr.s_addr = htonl(INADDR_ANY);
	sin.sin_family = AF_INET;
	sin.sin_port = htons(PORT);

	if (bind(sock, (SOCKADDR *) &sin, sizeof(sin)) == SOCKET_ERROR) {
		perror("bind()");
		exit(errno);
	}

	if (listen(sock, MAX_CONNEC) == SOCKET_ERROR) {
		perror("listen()");
		exit(errno);
	}

	return sock;
}

/* Ecoute sur le socket serveur les requetes de nouveaux clients et en accepte
 * automatiquement un. Retourne le socket client cree */
SOCKET server_socket_accept_client(SOCKET sock) {
	SOCKADDR_IN csin = { 0 };
	SOCKET csock = -1;
	int csinsize, ready = 0;
	fd_set readfs;
	struct timeval tv = { 0, 500 };

	csinsize = sizeof(csin);

	// Tant qu'un client n'a pas ete accepte, on continu de verifier si des
	// donnees sont a lire sur le socket
	while(!ready && !MGBT_EXIT_SIGNAL) {
		FD_ZERO(&readfs);
		FD_SET(sock, &readfs);

		if((ready = select(sock + 1, &readfs, NULL, NULL, &tv)) < 0) {
			perror("select()");
			exit(errno);
		}

		usleep(1);
	}

	// Si il y a des donnees a lire, alors c'est un client qui souhaite
	// relier le serveur, on l'accepte automatiquement
	if(ready && FD_ISSET(sock, &readfs)) {
		csock = accept(sock, (SOCKADDR *) &csin, (socklen_t *) &csinsize);

		if (csock == INVALID_SOCKET) {
			perror("accept()");
			exit(errno);
		}
	}

	return csock;
}

/* Lance le thread qui sera charge d'accueillir en continu tous les nouveaux
 * clients */
pthread_t server_socket_welcome_clients_start(Clients* clients, SOCKET sock,
		Series* series) {
	WelcomeClients* wClients;
	pthread_t wThread;

	clients->list = NULL;
	memset(&(clients->mutex), 0, sizeof(clients->mutex));

	// Structure de donnees pour communiquer avec la fonction threadee
	wClients = (WelcomeClients *) malloc(sizeof(WelcomeClients));
	wClients->clients = clients;
	wClients->series = series;
	wClients->sock = sock;

	pthread_create(&wThread, NULL, (void *) server_socket_welcome_clients,
			(void *) wClients);

	return wThread;
}

/* Arret du thread */
void server_socket_welcome_clients_stop(pthread_t wThread) {
	pthread_join(wThread, NULL);
}

/* Fonction threadee. Accepte en continu tous les nouveaux clients et les
 * ajoutent a la liste des clients. */
void server_socket_welcome_clients(void* p_data) {
	Client *client;
	SOCKET csock;
	WelcomeClients* wClients = (WelcomeClients *) p_data;

	// Tant que le jeu ne s'arrete pas, on accepte les nouveaux clients
	while (!MGBT_EXIT_SIGNAL) {

		// Un nouveau client vient d'arriver
		csock = server_socket_accept_client(wClients->sock);

		if(MGBT_EXIT_SIGNAL)
			break;

		// Creation du client
		client = (Client *) malloc(sizeof(Client));
		client->sock = csock;
		client->next = NULL;

		// Envoi du premier etat au client : il affichera l'ecran de
		// base, en attendant la prochaine mise a jour, dans laquelle il
		// sera automatiquement inclus. Ajout du client dans la liste.
		server_setWindowClient_first(client, wClients->clients, wClients->series);
		server_socket_add_client(wClients->clients, client);
		server_socket_send_windowClient(client, wClients->clients);
	}

	free(wClients);
}

/* Ajout d'un client dans la liste des clients (liste chainee). */
void server_socket_add_client(Clients* clients, Client* client) {
	Client* p_client = clients->list;

	pthread_mutex_lock(&(clients->mutex));

	// Ajout a la fin de la liste
	if (p_client != NULL) {

		// Recherche de la queue de la chaine
		while (p_client->next != NULL)
			p_client = p_client->next;

		// Ajout
		p_client->next = client;

	} else
		// La chaine est vide : premier client
		clients->list = client;

	clients->nb++;

	puts("GAME: +1 client");
	pthread_mutex_unlock(&(clients->mutex));
}

/* Suppression d'un client de la liste des clients (liste chainee). */
void server_socket_del_client(Clients* clients, Client* client) {
	Client *p_client, *p_client_bef = NULL;

	pthread_mutex_lock(&(clients->mutex));

	p_client = clients->list;

	// Recherche du client
	while (p_client != NULL && p_client != client) {
		p_client_bef = p_client;
		p_client = p_client->next;
	}

	// Le client a ete trouve : on le supprime
	if (p_client == client) {
		if (p_client_bef == NULL)
			clients->list = client->next;
		else
			p_client_bef->next = client->next;

		free(client);
	}

	clients->nb--;

	puts("GAME: -1 client");
	pthread_mutex_unlock(&(clients->mutex));
}

/* Envoi d'une structure de mise a jour de fenetre a un client en particulier */
void server_socket_send_windowClient(Client* client, Clients* clients) {
	if (send(client->sock, client->winClient, sizeof(WindowClient), 0) < 0)
		server_socket_del_client(clients, client);
}

/* Envoi des structures de mise a jour des fenetres respectives des differents
 * clients */
void server_socket_send_windowClients(Clients* clients) {
	Client* client = clients->list;

	while (client != NULL) {
		server_socket_send_windowClient(client, clients);
		client = client->next;
	}
}

/* Nombre de clients */
int server_socket_count_clients(Clients* clients) {
	return clients->nb;
}

/* Liberation de l'espace alloue aux clients */
void server_socket_free(Clients* clients) {
	Client *client, *p_client = clients->list;

	pthread_mutex_lock(&(clients->mutex));

	while (p_client != NULL) {
		client = p_client;
		p_client = p_client->next;

		free(client->winClient);

		// Liberation et fermeture des sockets
		server_socket_close_client(client);
		server_socket_free_client(client);
	}

	pthread_mutex_unlock(&(clients->mutex));
}
