#include "server_sound.h"

/* Lancement d'un thread qui aura la capacite de lancer du son en parallele du
 * programme principal. */
pthread_t server_sound_start(SoundData* data) {
	pthread_t sThread;

	data->status = SOUND_STOP;

	pthread_create(&sThread, NULL, (void *) server_sound,
			(void *) data);

	return sThread;
}

/* Fonction threadee. Verifie en continue l'etat de la structure de donnees
 * passee en parametre, pour lancer ou non un fichier musical. */
void server_sound(void* p_data) {
	SoundData* data = (SoundData *) p_data;
	SoundGlobals soundGlobals;

	alutInit(NULL, NULL);

	// Tant que le jeu n'est pas arrete
	while(!MGBT_EXIT_SIGNAL) {

		// On attend d'avoir l'ordre de jouer ce qui est dans la
		// structure
		while(data->status != SOUND_PLAY && !MGBT_EXIT_SIGNAL)
			usleep(1);

		if(MGBT_EXIT_SIGNAL)
			break;

		// Ouverture du fichier audio
		server_sound_open(data->filepath, &soundGlobals);

		if(!server_sound_playback(&soundGlobals)) {
			perror("Ogg refused to play.");
			exit(1);
		}

		// Mise a jour en continu des donnees audio a lire (lecture en
		// streaming)
		while(data->status != SOUND_STOP && !MGBT_EXIT_SIGNAL && server_sound_update(&soundGlobals)) {
			if(data->status == SOUND_PAUSE) {
				alSourcePause(soundGlobals.source);

				// Une pause a ete demandee : on attend qu'elle
				// soit annulee
				while(data->status == SOUND_PAUSE && !MGBT_EXIT_SIGNAL)
					usleep(1);

				if(!MGBT_EXIT_SIGNAL)
					alSourcePlay(soundGlobals.source);
			}

			usleep(1);
		}

		data->status = SOUND_STOP;

		server_sound_release(&soundGlobals);
	}

	server_sound_shutdownOpenAL();

	alutExit();
}

/* Arret du thread */
void server_sound_stop(pthread_t sThread) {
	pthread_join(sThread, NULL);
}

/* On souhaite que le thread se mette a lire un generique de serie passee en
 * parametre */
void server_sound_play_start_serie(SoundData* data, char* serieDir) {
	sprintf(data->filepath, "data/series/%s/sound.ogg", serieDir);
	data->status = SOUND_PLAY;
}

/* On souhaite lire un son qui n'est pas un generique */
void server_sound_play_start_events(SoundData* data, char* event) {
	sprintf(data->filepath, "data/sounds/%s.ogg", event);
	data->status = SOUND_PLAY;
}

/* Mise en pause */
void server_sound_play_pause(SoundData* data) {
	data->status = SOUND_PAUSE;
}

/* Reprise de la pause */
void server_sound_play_reprise(SoundData* data) {
	data->status = SOUND_PLAY;
}

/* Arret de la lecture */
void server_sound_play_stop(SoundData* data) {
	data->status = SOUND_STOP;
}

/* Choix d'une serie au hasard pour lancer le generique */
char* server_sound_random(Series* series) {
	struct timespec ts;

	clock_gettime(CLOCK_REALTIME, &ts);
	srand(ts.tv_sec + ts.tv_nsec);

	return series->list[rand() % series->nb_serie]->dir;
}


/* Code from http://www.devmaster.net/articles/openal-tutorials/lesson8.php */

void server_sound_open(char* path, SoundGlobals* soundGlobals) {
	FILE* oggFile;
	int result;

	if(!(oggFile = fopen(path, "rb"))) {
		perror("Could not open Ogg file.");
		exit(1);
	}

	if((result = ov_open(oggFile, &soundGlobals->oggStream, NULL, 0)) < 0) {
		fclose(oggFile);

		perror("Could not open Ogg stream.");
		exit(1);
	}

	soundGlobals->vorbisInfo = ov_info(&soundGlobals->oggStream, -1);
	soundGlobals->vorbisComment = ov_comment(&soundGlobals->oggStream, -1);

	if(soundGlobals->vorbisInfo->channels == 1)
		soundGlobals->format = AL_FORMAT_MONO16;
	else
		soundGlobals->format = AL_FORMAT_STEREO16;

	alGenBuffers(2, soundGlobals->buffers);

	server_sound_check();
	alGenSources(1, &soundGlobals->source);
	server_sound_check();

	alSource3f(soundGlobals->source, AL_POSITION,        0.0, 0.0, 0.0);
	alSource3f(soundGlobals->source, AL_VELOCITY,        0.0, 0.0, 0.0);
	alSource3f(soundGlobals->source, AL_DIRECTION,       0.0, 0.0, 0.0);
	alSourcef (soundGlobals->source, AL_ROLLOFF_FACTOR,  0.0          );
	alSourcei (soundGlobals->source, AL_SOURCE_RELATIVE, AL_TRUE      );
}

// Fixed
void server_sound_release(SoundGlobals* soundGlobals) {
	alSourceStop(soundGlobals->source);
	server_sound_empty(soundGlobals);
	alDeleteSources(1, &soundGlobals->source);
	server_sound_check();
	alDeleteBuffers(2, soundGlobals->buffers);
	server_sound_check();

	ov_clear(&soundGlobals->oggStream);
}

int server_sound_playback(SoundGlobals* soundGlobals) {
	if(server_sound_playing(soundGlobals))
		return 1;

	if(!server_sound_stream(soundGlobals->buffers[0], soundGlobals))
		return 0;

	if(!server_sound_stream(soundGlobals->buffers[1], soundGlobals))
		return 0;

	alSourceQueueBuffers(soundGlobals->source, 2, soundGlobals->buffers);
	alSourcePlay(soundGlobals->source);

	return 1;
}

int server_sound_playing(SoundGlobals* soundGlobals) {
	ALenum state;

	alGetSourcei(soundGlobals->source, AL_SOURCE_STATE, &state);

	return (state == AL_PLAYING);
}

int server_sound_update(SoundGlobals* soundGlobals) {
	int processed;
	int active = 1;

	alGetSourcei(soundGlobals->source, AL_BUFFERS_PROCESSED, &processed);

	while(processed-- && active) {
		ALuint buffer;

		alSourceUnqueueBuffers(soundGlobals->source, 1, &buffer);
		server_sound_check();

		active = server_sound_stream(buffer, soundGlobals);

		alSourceQueueBuffers(soundGlobals->source, 1, &buffer);
		server_sound_check();
	}

	return active;
}

int server_sound_stream(ALuint buffer, SoundGlobals* soundGlobals) {
	char pcm[BUFFER_SIZE];
	int  size = 0;
	int  section;
	int  result;

	while(size < BUFFER_SIZE) {
		result = ov_read(&soundGlobals->oggStream, pcm + size, BUFFER_SIZE - size, 0, 2, 1, &section);

		if(result > 0)
			size += result;
		else
			if(result < 0) {
				perror("Stream error.");
				exit(1);
			} else
				break;
	}

	if(result == 0)
		return 0;

	alBufferData(buffer, soundGlobals->format, pcm, size, soundGlobals->vorbisInfo->rate);
	server_sound_check();

	return 1;
}

void server_sound_empty(SoundGlobals* soundGlobals) {
	int queued;

	alGetSourcei(soundGlobals->source, AL_BUFFERS_QUEUED, &queued);

	while(queued--) {
		ALuint buffer;

		alSourceUnqueueBuffers(soundGlobals->source, 1, &buffer);
		server_sound_check();
	}
}

void server_sound_check() {
	int error = alGetError();

	if(error != AL_NO_ERROR) {
		char str[256];
		sprintf(str, "OpenAL error was raised : 0x%x\n", error);

		perror(str);
		exit(1);
	}
}

/** Code from http://loulou.developpez.com/tutoriels/openal/premiers-pas **/

void server_sound_shutdownOpenAL() {
	ALCcontext* context = alcGetCurrentContext();
	ALCdevice*  device  = alcGetContextsDevice(context);

	alcDestroyContext(context);
	alcCloseDevice(device);
}
