#include "server_wiimote.h"

// static int g_banner = 1; dans src/wiiuse.c

/* Detection des wiimotes, et initialisation des joueurs correspondants. */
void server_wiimote_init(GameStatus* game) {
	wiimote** wiimotes;
	int found, i;
	int leds[] = { WIIMOTE_LED_1, WIIMOTE_LED_2, WIIMOTE_LED_3, WIIMOTE_LED_4 };

	// Detection des wiimotes
	wiimotes = wiiuse_init(MAX_WIIMOTES);
	found = wiiuse_find(wiimotes, MAX_WIIMOTES, WIIMOTE_DETECT_TIMEOUT);

	// Des wiimotes ont ete detectees
	if (found) {

		// Nombre de wiimotes
		game->nb_players = wiiuse_connect(wiimotes, MAX_WIIMOTES);

		// Creation des joueurs en leur attribuant une wiimote
		for (i = 0; i < game->nb_players; i++) {
			game->players[i].wiimote = wiimotes[i];
			game->players[i].wiimoteStatus = W_INIT;
			game->players[i].id = i;

			// Chaque wiimote est configuree, et affiche la led
			// correspondant a leur numero de joueur dans la partie
			wiiuse_motion_sensing(wiimotes[i], 0);
			wiiuse_set_ir(wiimotes[i], 0);
			wiiuse_set_leds(wiimotes[i], leds[i]);

			// Chaque detectee se met a vibro un coup pour
			// confirmer
			wiiuse_rumble(wiimotes[i], 1);
		}

		usleep(100000);

		// Fin de la vibration des wiimotes
		for (i = 0; i < game->nb_players; i++)
			wiiuse_rumble(wiimotes[i], 0);
	}
}

/* Lancement du thread charge de recuperer les evenements wiimote. */
pthread_t server_wiimote_start(GameStatus* game) {
	pthread_t wThread;

	pthread_create(&wThread, NULL, (void *) server_wiimote, (void *) game);

	return wThread;
}

/* Fonction threadee. Demande la recuperation dans les cache des wiimote de la
 * derniere action enregistree et les parse. */
void server_wiimote(void* p_data) {
	GameStatus* game = (GameStatus *) p_data;
	wiimote** wiimotes;
	int i = 0, alive = 1;

	// wiiuse_poll necessite un tableau des wiimotes
	wiimotes = (wiimote**) malloc(sizeof(wiimote) * game->nb_players);
	for (i = 0; i < game->nb_players; i++) {
		wiimotes[i] = game->players[i].wiimote;
	}

	// Jusqu'a ce que le jeu s'arrete ou qu'il n'y ai plus aucune wiimote
	// accessible
	while (alive && !MGBT_EXIT_SIGNAL) {

		// Mise a jour du cache des evenements wiimote (depile leur
		// buffer)
		if (wiiuse_poll(wiimotes, game->nb_players)) {
			alive = 0;

			// Pour chacun des joueurs/wiimote
			for (i = 0; i < game->nb_players && !MGBT_EXIT_SIGNAL; i++)
				if (game->players[i].wiimoteStatus != W_DISCONNECT) {
					alive = 1;

					// Tant qu'un evenement est associe a sa
					// structure de donnees, on attent
					// qu'elle soit traitee par le programme
					// principal
					while (game->players[i].hasNewEvent && !MGBT_EXIT_SIGNAL)
						usleep(1);

					// Mise a jour du joueur en fonction de
					// l'evenement detecte
					server_wiimote_action(&(game->players[i]));
				}
		}

		usleep(1);
	}

	if (!alive)
		server_quit_all();

	wiiuse_cleanup(wiimotes, game->nb_players);
}

/* Arret du thread. */
void server_wiimote_stop(pthread_t wThread) {
	pthread_join(wThread, NULL);
}

/* Mise a jour du joueur en fonction de l'evenement wiimote qui a ete detecte */
void server_wiimote_action(PlayerStatus* player) {

	// Recherche de l'evenement wiimote qui a ete detecte
	switch (player->wiimote->event) {

	// Evenement "classique"
	case WIIUSE_EVENT:

		// Bouton A presse : demande de l'arret du jeu
		if (IS_PRESSED(player->wiimote, WIIMOTE_BUTTON_A)) {
			player->wiimoteStatus = W_QUIT;
			player->hasNewEvent = 1;

			// Sinon, si il a bien une GH3 de connectee
		} else if (player->wiimote->exp.type == EXP_GUITAR_HERO_3) {
			gh3_t* gh3 = (gh3_t *) &(player->wiimote->exp.gh3);

			// Si le jeu est en pause, on ne traite que le cas de la
			// demande de reprise du jeu
			if (player->wiimoteStatus == W_PAUSE) {
				if (IS_PRESSED(gh3, GUITAR_HERO_3_BUTTON_PLUS)
						|| IS_PRESSED(gh3, GUITAR_HERO_3_BUTTON_MINUS)) {
					player->wiimoteStatus = W_REPRISE;
					player->hasNewEvent = 1;
				}

				// Sinon on met a jour la structure de donneees
			} else if (player->wiimoteStatus != W_NOEVENT) {

				// + ou - de la guitare a ete presse : on
				// reclame la pause au serveur
				if (IS_PRESSED(gh3, GUITAR_HERO_3_BUTTON_PLUS)
						|| IS_PRESSED(gh3, GUITAR_HERO_3_BUTTON_MINUS)) {
					player->wiimoteStatus = W_REQ_PAUSE;
					player->hasNewEvent = 1;
				}

				// Un coup de strum a ete donne : on regarde
				// la (ou les) couleur qui est simultanement
				// pressee, et on incremente ses coups de strums
				else if (IS_PRESSED(gh3, GUITAR_HERO_3_BUTTON_STRUM_UP)
						|| IS_PRESSED(gh3, GUITAR_HERO_3_BUTTON_STRUM_DOWN)) {

					if (IS_PRESSED(gh3, GUITAR_HERO_3_BUTTON_GREEN))
						player->colors[G_GREEN].nbStrums++;

					if (IS_PRESSED(gh3, GUITAR_HERO_3_BUTTON_RED))
						player->colors[G_RED].nbStrums++;

					if (IS_PRESSED(gh3, GUITAR_HERO_3_BUTTON_YELLOW))
						player->colors[G_YELLOW].nbStrums++;

					if (IS_PRESSED(gh3, GUITAR_HERO_3_BUTTON_BLUE))
						player->colors[G_BLUE].nbStrums++;

					if (IS_PRESSED(gh3, GUITAR_HERO_3_BUTTON_ORANGE))
						player->colors[G_ORANGE].nbStrums++;

					// Sinon, c'est peut-etre un relachement de
					// couleur (validation d'une serie de strums).
					// On verifie pour chacun, et on signal son
					// activation si elle vient d'un relachee
				} else {

					if (IS_RELEASED(gh3, GUITAR_HERO_3_BUTTON_GREEN)) {
						player->colors[G_GREEN].validated = 1;
						player->hasNewEvent = 1;
					}

					if (IS_RELEASED(gh3, GUITAR_HERO_3_BUTTON_RED)) {
						player->colors[G_RED].validated = 1;
						player->hasNewEvent = 1;
					}

					if (IS_RELEASED(gh3, GUITAR_HERO_3_BUTTON_YELLOW)) {
						player->colors[G_YELLOW].validated = 1;
						player->hasNewEvent = 1;
					}

					if (IS_RELEASED(gh3, GUITAR_HERO_3_BUTTON_BLUE)) {
						player->colors[G_BLUE].validated = 1;
						player->hasNewEvent = 1;
					}

					if (IS_RELEASED(gh3, GUITAR_HERO_3_BUTTON_ORANGE)) {
						player->colors[G_ORANGE].validated = 1;
						player->hasNewEvent = 1;
					}

					if (gh3->whammy_bar > 0.5 && player->wiimoteStatus
							!= W_WHAMMY_DOWN) {
						player->colors[G_WHAMMY_BAR].validated = 1;
						player->hasNewEvent = 1;
						player->wiimoteStatus = W_WHAMMY_DOWN;
					}

					if (gh3->whammy_bar == 0)
						player->wiimoteStatus = W_PLAY;
				}
			}
		}
		break;

		// La wiimote vient de se deconnecter
	case WIIUSE_DISCONNECT:
	case WIIUSE_UNEXPECTED_DISCONNECT:
		player->wiimoteStatus = W_DISCONNECT;
		player->hasNewEvent = 1;

		puts("GAME: -1 player");
		break;

		// La GH3 vient d'etre enclenchee
	case WIIUSE_GUITAR_HERO_3_CTRL_INSERTED:
		player->wiimoteStatus = W_PLAY;

		puts("GAME: +1 player");
		break;

		// On ignore les autres cas
	default:
		break;
	}
}

/* Indique au joueur qu'il a bien joue en vivrant */
void server_wiimote_good(PlayerStatus* player) {
	wiiuse_rumble(player->wiimote, 1);
	usleep(200000);
	wiiuse_rumble(player->wiimote, 0);
	usleep(200000);
	wiiuse_rumble(player->wiimote, 1);
	usleep(200000);
	wiiuse_rumble(player->wiimote, 0);
}

/* Indique au joueur que son coup de vibrato etait mauvais, en vibrant */
void server_wiimote_badWhammyBar(PlayerStatus* player) {
	wiiuse_rumble(player->wiimote, 1);
	usleep(800000);
	wiiuse_rumble(player->wiimote, 0);
}

/* Indique au joueur qu'il a mal joue, en vibrant */
void server_wiimote_bad(PlayerStatus* player) {
	wiiuse_rumble(player->wiimote, 1);
	usleep(500000);
	wiiuse_rumble(player->wiimote, 0);
}

/* Indique aux wiimotes d'ignorer tous les evenements (durant la pause, par
 * exemple), ce qui permet de faire vider leur buffer sans le traiter. */
void server_wiimote_lockEvents(GameStatus* game) {
	int p;

	for (p = 0; p < game->nb_players; p++)
		game->players[p].wiimoteStatus = W_NOEVENT;
}

/* Indique qu'il est de nouveau necessaire de traiter les evenements wiimotes */
void server_wiimote_unlockEvents(GameStatus* game) {
	int p;

	for (p = 0; p < game->nb_players; p++)
		game->players[p].wiimoteStatus = W_PLAY;
}
